@@ -8,7 +8,7 @@
 
    This IR uses parametrized basic blocks (in contrast to phi nodes).
 
-   The core of the IR is the `Instruction<T, K>` template,
+   The core of the IR is the `Instruction<I, A>` template,
    where `T` denotes the set of intrinsics (user-defined)
    and `K` denotes the set of attributes (static information
    about instructions) which is also user-defined.
@@ -30,152 +30,102 @@
 
 */
 
-// Supports serialization to (compressed) bincode.
+use alloc::string::String;
+use alloc::vec::Vec;
 use serde::{Deserialize, Serialize};
-use std::fmt;
-
-/////
-///// ExtIR - an IR specialized to represent function-like dataflow.
-/////
 
 #[derive(Clone, Debug)]
 pub enum IRError {
     Fallback,
 }
 
-#[derive(Serialize, Deserialize, PartialEq, Eq, Hash, Copy, Clone, Debug)]
-pub struct Var {
-    pub id: usize,
-}
+#[derive(Clone, Debug, Copy, PartialEq, Eq, PartialOrd, Ord, Hash, Serialize, Deserialize)]
+pub struct Var(usize);
 
-pub fn var(id: usize) -> Var {
-    Var { id }
-}
-
-impl fmt::Display for Var {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "%{}", self.id)
+impl Var {
+    pub fn get_id(&self) -> usize {
+        self.0
     }
 }
 
-#[derive(Serialize, Deserialize, Clone, Debug)]
-pub enum Operator<T> {
-    Intrinsic(T),
-    ModuleRef(Vec<String>, String),
+#[derive(Clone, Debug, Serialize, Deserialize)]
+pub enum Operator<I> {
+    Intrinsic(I),
+    ModuleRef(Option<String>, String),
 }
 
-impl<T> fmt::Display for Operator<T>
-where
-    T: fmt::Display,
-{
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        match self {
-            Operator::Intrinsic(v) => write!(f, "{}", v),
-            Operator::ModuleRef(module, name) => {
-                for k in module.iter() {
-                    write!(f, "{}.", k)?;
-                }
-                write!(f, "@{}", name)
-            }
-        }
+#[derive(Clone, Debug, Serialize, Deserialize)]
+pub struct Instruction<I, A> {
+    op: Operator<I>,
+    args: Vec<Var>,
+    attrs: Vec<A>,
+}
+
+impl<I, A> Instruction<I, A> {
+    pub fn new(op: Operator<I>, args: Vec<Var>, attrs: Vec<A>) -> Instruction<I, A> {
+        Instruction { op, args, attrs }
+    }
+
+    pub fn get_op(&self) -> &Operator<I> {
+        &self.op
+    }
+
+    pub fn get_args(&self) -> &[Var] {
+        &self.args
+    }
+
+    pub fn get_attrs(&self) -> &[A] {
+        &self.attrs
     }
 }
 
-#[derive(Serialize, Deserialize, Clone, Debug)]
-pub enum BranchCondition {
-    None,
-    Some(Var),
-}
-
-#[derive(Serialize, Deserialize, Clone, Debug)]
+#[derive(Clone, Debug, Serialize, Deserialize)]
 pub struct Branch {
-    cond: BranchCondition,
+    cond: Option<Var>,
     block: usize,
     args: Vec<Var>,
 }
 
-impl fmt::Display for Branch {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "  br {} (", self.block)?;
-        let l = self.args.len();
-        for (ind, arg) in self.args.iter().enumerate() {
-            match l - 1 == ind {
-                true => write!(f, "{}", arg)?,
-                _ => write!(f, "{}, ", arg)?,
-            };
-        }
-        write!(f, ")")?;
-        match self.cond {
-            BranchCondition::None => Ok(()),
-            BranchCondition::Some(v) => write!(f, " if {}", v),
-        }
-    }
-}
-
 impl Branch {
-    fn arguments(&mut self) -> &mut Vec<Var> {
-        &mut self.args
+    pub fn get_args(&self) -> &[Var] {
+        &self.args
     }
 
-    pub fn block(&self) -> usize {
+    pub fn get_block(&self) -> usize {
         self.block
     }
 
-    fn isconditional(&self) -> bool {
+    pub fn is_conditional(&self) -> bool {
         match self.cond {
-            BranchCondition::Some(_v) => true,
-            BranchCondition::None => false,
+            Some(_v) => true,
+            None => false,
         }
     }
 }
 
-#[derive(Serialize, Deserialize, Clone, Debug)]
-pub struct Instruction<T, K> {
-    pub op: Operator<T>,
-    pub args: Vec<Var>,
-    pub attrs: Vec<K>,
+#[derive(Clone, Debug, Serialize, Deserialize)]
+pub struct BasicBlock<I, A> {
+    args: Vec<Var>,
+    insts: Vec<Instruction<I, A>>,
+    branches: Vec<Branch>,
 }
 
-impl<T, K> fmt::Display for Instruction<T, K>
-where
-    T: fmt::Display,
-    K: fmt::Display,
-{
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "{}", self.op)?;
-        write!(f, "(")?;
-        let l = self.args.len();
-        for (ind, arg) in self.args.iter().enumerate() {
-            match l - 1 == ind {
-                true => write!(f, "{}", arg)?,
-                _ => write!(f, "{}, ", arg)?,
-            };
-        }
-        write!(f, ")")?;
-        if !self.attrs.is_empty() {
-            write!(f, " {{ ")?;
-            let l = self.attrs.len();
-            for (ind, attr) in self.attrs.iter().enumerate() {
-                match l - 1 == ind {
-                    true => write!(f, "{}", attr)?,
-                    _ => write!(f, "{}, ", attr)?,
-                };
-            }
-            write!(f, " }}")?;
-        }
-        Ok(())
+impl<I, A> BasicBlock<I, A> {
+    pub fn get_insts(&self) -> &[Instruction<I, A>] {
+        &self.insts
+    }
+
+    pub fn get_args(&self) -> &[Var] {
+        &self.args
+    }
+
+    pub fn get_branches(&self) -> &[Branch] {
+        &self.branches
     }
 }
 
-#[derive(Serialize, Deserialize, Clone, Debug)]
-pub struct BasicBlock<T, K> {
-    insts: Vec<Instruction<T, K>>,
-    pub args: Vec<Var>,
-    pub branches: Vec<Branch>,
-}
-
-impl<T, K> Default for BasicBlock<T, K> {
-    fn default() -> BasicBlock<T, K> {
+impl<I, A> Default for BasicBlock<I, A> {
+    fn default() -> BasicBlock<I, A> {
         BasicBlock {
             insts: Vec::new(),
             args: Vec::new(),
@@ -184,22 +134,22 @@ impl<T, K> Default for BasicBlock<T, K> {
     }
 }
 
-#[derive(Serialize, Deserialize, Clone, Debug)]
+#[derive(Clone, Debug, Serialize, Deserialize)]
 pub struct IRLInfo {
     file: String,
     module: String,
     line: usize,
 }
 
-#[derive(Serialize, Deserialize, Clone, Debug)]
-pub struct ExtIR<T, K> {
+#[derive(Clone, Debug, Serialize, Deserialize)]
+pub struct ExtIR<I, A> {
     defs: Vec<(i32, i32)>,
-    pub blocks: Vec<BasicBlock<T, K>>,
-    lines: Vec<IRLInfo>,
+    lines: Vec<Option<IRLInfo>>,
+    blocks: Vec<BasicBlock<I, A>>,
 }
 
-impl<T, K> Default for ExtIR<T, K> {
-    fn default() -> ExtIR<T, K> {
+impl<I, A> Default for ExtIR<I, A> {
+    fn default() -> ExtIR<I, A> {
         let entry = BasicBlock::default();
         ExtIR {
             defs: Vec::new(),
@@ -209,27 +159,29 @@ impl<T, K> Default for ExtIR<T, K> {
     }
 }
 
-impl<T, K> ExtIR<T, K> {
-    pub fn get_args(&self) -> Vec<Var> {
-        let b = &self.blocks[0];
-        b.args.to_vec()
+impl<I, A> ExtIR<I, A> {
+    pub fn get_args(&self) -> &[Var] {
+        self.blocks[0].get_args()
+    }
+
+    pub fn get_block_args(&self, blk: usize) -> &[Var] {
+        self.blocks[blk].get_args()
     }
 
     pub fn push_arg(&mut self, blk: usize) -> Var {
-        let arg = var(self.defs.len());
+        let arg = Var(self.defs.len());
         self.defs.push((blk as i32, -1));
         self.blocks[blk].args.push(arg);
         arg
     }
 
-    pub fn push_block(&mut self) -> usize {
+    pub fn push_blk(&mut self) -> usize {
         let blk = BasicBlock::default();
         self.blocks.push(blk);
-
         self.blocks.len() - 1
     }
 
-    pub fn get_branches(&self, blk: usize) -> &Vec<Branch> {
+    pub fn get_branches(&self, blk: usize) -> &[Branch] {
         &self.blocks[blk].branches
     }
 
@@ -237,17 +189,14 @@ impl<T, K> ExtIR<T, K> {
         &mut self.blocks[blk].branches
     }
 
-    pub fn push_branch<'b>(
-        &mut self,
-        cond: BranchCondition,
-        from: usize,
-        to: usize,
-        args: Vec<Var>,
-    ) {
+    /// Push a branch from the block indexed by `from`
+    /// to the branch indexed by `to`.
+    /// The user must supply a branch condition in `cond`.
+    pub fn push_branch(&mut self, cond: Option<Var>, from: usize, to: usize, args: Vec<Var>) {
         let brs = self.get_branches_mut(from);
         match cond {
-            BranchCondition::None => brs.retain(|x| x.isconditional()),
-            BranchCondition::Some(_v) => (),
+            None => brs.retain(|x| x.is_conditional()),
+            Some(_v) => (),
         }
         brs.push(Branch {
             cond,
@@ -256,7 +205,10 @@ impl<T, K> ExtIR<T, K> {
         });
     }
 
-    pub fn get_instr(&self, id: Var) -> Option<(Var, &Instruction<T, K>)> {
+    /// Get an immutable reference to a "line" of the IR.
+    /// The IR is indexed with `id` (a `Var` instance).
+    /// This returns `(Var, &Instruction<I, A>)`.
+    pub fn get_instr(&self, id: Var) -> Option<(Var, &Instruction<I, A>)> {
         match self.get_var_blockidx(id) {
             None => None,
             Some((b, i)) => {
@@ -267,8 +219,10 @@ impl<T, K> ExtIR<T, K> {
         }
     }
 
-    pub fn push_instr(&mut self, blk: usize, v: Instruction<T, K>) -> Var {
-        let arg = var(self.defs.len());
+    /// Push an instruction onto the IR at block index `blk`.
+    /// Returns a new `Var` reference to that instruction.
+    pub fn push_instr(&mut self, blk: usize, v: Instruction<I, A>) -> Var {
+        let arg = Var(self.defs.len());
         let len = self.blocks[blk].insts.len();
         let bb = &mut self.blocks[blk];
         bb.insts.push(v);
@@ -276,7 +230,7 @@ impl<T, K> ExtIR<T, K> {
         arg
     }
 
-    fn get_instr_mut(&mut self, id: Var) -> Option<(Var, &mut Instruction<T, K>)> {
+    fn get_instr_mut(&mut self, id: Var) -> Option<(Var, &mut Instruction<I, A>)> {
         match self.get_var_blockidx(id) {
             None => None,
             Some((b, i)) => {
@@ -287,18 +241,20 @@ impl<T, K> ExtIR<T, K> {
         }
     }
 
-    pub fn get_block_vars(&self, id: usize) -> Vec<Var> {
+    /// Get the vector of `Var` which index into block with index `id`.
+    fn get_block_vars(&self, id: usize) -> Vec<Var> {
         let v = self
             .defs
             .iter()
             .enumerate()
             .filter(|(_, v)| v.0 == (id as i32) && v.1 >= 0);
-        let mut m = v.map(|(i, l)| (var(i), l)).collect::<Vec<_>>();
+        let mut m = v.map(|(i, l)| (Var(i), l)).collect::<Vec<_>>();
         m.sort_by(|a, b| a.1.cmp(b.1));
         m.iter().map(|v| v.0).collect::<Vec<_>>()
     }
 
-    pub fn block_iter(&self, id: usize) -> ImmutableBlockIterator<T, K> {
+    /// Get an immutable iterator over basic blocks.
+    fn block_iter(&self, id: usize) -> ImmutableBlockIterator<I, A> {
         let ks = self.get_block_vars(id);
         ImmutableBlockIterator {
             ir: self,
@@ -307,7 +263,8 @@ impl<T, K> ExtIR<T, K> {
         }
     }
 
-    pub fn block_iter_mut(&mut self, id: usize) -> MutableBlockIterator<T, K> {
+    /// Get a mutable iterator over basic blocks.
+    fn block_iter_mut(&mut self, id: usize) -> MutableBlockIterator<I, A> {
         let ks = self.get_block_vars(id);
         MutableBlockIterator {
             ir: self,
@@ -316,8 +273,9 @@ impl<T, K> ExtIR<T, K> {
         }
     }
 
+    /// Get the block index and SSA index for `v: Var`.
     fn get_var_blockidx(&self, v: Var) -> Option<(usize, i32)> {
-        let (b, i) = self.defs.get(v.id).unwrap_or(&(-1, -1));
+        let (b, i) = self.defs.get(v.get_id()).unwrap_or(&(-1, -1));
         if *i < 0 {
             None
         } else {
@@ -328,7 +286,7 @@ impl<T, K> ExtIR<T, K> {
     pub fn get_vars_in_block(&self, blockidx: usize) -> Vec<Var> {
         let mut v: Vec<Var> = Vec::new();
         for ind in 0..self.defs.len() {
-            let r = Var { id: ind };
+            let r = Var(ind);
             match self.get_var_blockidx(r) {
                 None => (),
                 Some(b) => {
@@ -349,7 +307,7 @@ impl<T, K> ExtIR<T, K> {
                     let br = &b.branches[bi];
                     if br.block >= i {
                         b.branches[bi] = Branch {
-                            cond: br.cond.clone(),
+                            cond: br.cond,
                             block: br.block - 1,
                             args: br.args.clone(),
                         };
@@ -359,66 +317,17 @@ impl<T, K> ExtIR<T, K> {
         }
 
         for (_, (b, j)) in self.defs.iter_mut().enumerate() {
-            if *b == i as i32 {
-                *b = -1;
-                *j = -1;
-            } else if *b > i as i32 {
-                *b -= i as i32;
-            }
-        }
-    }
-}
-
-impl<T, K> fmt::Display for ExtIR<T, K>
-where
-    T: fmt::Display,
-    K: fmt::Display,
-{
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        for ind in 0..self.blocks.len() {
-            write!(f, "{}: ", ind)?;
-            let b = &self.blocks[ind];
-            let bargs = &b.args;
-            if !bargs.is_empty() {
-                write!(f, "(")?;
-                let l = bargs.len();
-                for (ind, arg) in bargs.iter().enumerate() {
-                    match l - 1 == ind {
-                        true => write!(f, "{}", arg)?,
-                        _ => write!(f, "{}, ", arg)?,
-                    };
+            match *b == i as i32 {
+                true => {
+                    *b = -1;
+                    *j = -1;
                 }
-                write!(f, ")")?;
-            }
-            writeln!(f)?;
-            for (v, instr) in self.block_iter(ind) {
-                writeln!(f, "  {} = {}", v, instr)?;
-            }
-            for br in &self.blocks[ind].branches {
-                writeln!(f, "{}", br)?;
+                false => match *b > i as i32 {
+                    true => *b -= i as i32,
+                    false => (),
+                },
             }
         }
-        Ok(())
-    }
-}
-
-#[cfg(test)]
-mod extir_tests {
-    use super::*;
-
-    #[derive(Debug)]
-    enum FakeIntrinsic {
-        Null,
-    }
-
-    #[derive(Debug)]
-    enum FakeAttribute {
-        Null,
-    }
-
-    #[test]
-    fn test_construction() {
-        let _ir = ExtIR::<FakeIntrinsic, FakeAttribute>::default();
     }
 }
 
@@ -427,14 +336,14 @@ mod extir_tests {
 /////
 
 #[derive(Debug)]
-pub struct MutableBlockIterator<'b, T, K> {
-    ir: &'b mut ExtIR<T, K>,
+pub struct MutableBlockIterator<'b, I, A> {
+    ir: &'b mut ExtIR<I, A>,
     ks: Vec<Var>,
     state: usize,
 }
 
-impl<'b, T, K> Iterator for MutableBlockIterator<'b, T, K> {
-    type Item = (Var, &'b mut Instruction<T, K>);
+impl<'b, I, A> Iterator for MutableBlockIterator<'b, I, A> {
+    type Item = (Var, &'b mut Instruction<I, A>);
 
     fn next(&mut self) -> Option<Self::Item> {
         if self.state >= self.ks.len() {
@@ -446,7 +355,7 @@ impl<'b, T, K> Iterator for MutableBlockIterator<'b, T, K> {
                 None => None,
                 Some((v, inst)) => {
                     // somewhat convinced this is required.
-                    let ptr: *mut Instruction<T, K> = inst;
+                    let ptr: *mut Instruction<I, A> = inst;
                     Some((v, unsafe { &mut *ptr }))
                 }
             }
@@ -455,14 +364,14 @@ impl<'b, T, K> Iterator for MutableBlockIterator<'b, T, K> {
 }
 
 #[derive(Debug)]
-pub struct ImmutableBlockIterator<'b, T, K> {
-    ir: &'b ExtIR<T, K>,
+pub struct ImmutableBlockIterator<'b, I, A> {
+    ir: &'b ExtIR<I, A>,
     ks: Vec<Var>,
     state: usize,
 }
 
-impl<'b, T, K> Iterator for ImmutableBlockIterator<'b, T, K> {
-    type Item = (Var, &'b Instruction<T, K>);
+impl<'b, I, A> Iterator for ImmutableBlockIterator<'b, I, A> {
+    type Item = (Var, &'b Instruction<I, A>);
 
     fn next(&mut self) -> Option<Self::Item> {
         if self.ks.len() > self.state {
@@ -474,6 +383,19 @@ impl<'b, T, K> Iterator for ImmutableBlockIterator<'b, T, K> {
     }
 }
 
+/////
+///// Verification.
+/////
+
+/*
+
+   The verification pass checks that:
+
+   1. Uses are always dominated by a def.
+   2. Basic blocks are in extended form
+
+*/
+
 /////
 ///// Lowering.
 /////
@@ -488,7 +410,7 @@ impl<'b, T, K> Iterator for ImmutableBlockIterator<'b, T, K> {
 pub trait Lowering<T> {
     type IRBuilder;
     type Error;
-    fn prepare_builder(&self) -> Result<Self::IRBuilder, Self::Error>;
+    fn prepare(&self) -> Result<Self::IRBuilder, Self::Error>;
     fn build(&self, b: &mut Self::IRBuilder) -> Result<(), Self::Error>;
     fn lower(&self) -> Result<T, Self::Error>;
 }
@@ -520,14 +442,9 @@ pub trait Lowering<T> {
 // `R` encodes the return analysis type -- it's a contract
 // specified for `self.result()`.
 pub trait AbstractInterpreter<IR, R> {
-    type Error;
-    type LatticeElement;
-
-    // This is basically "state required to prepare an
-    // interpreter, before `step`".
-    //
-    // For typing, this might be type annotations.
     type Meta;
+    type LatticeElement;
+    type Error;
 
     fn prepare(meta: Self::Meta, ir: &IR) -> Result<Self, Self::Error>
     where
@@ -537,3 +454,112 @@ pub trait AbstractInterpreter<IR, R> {
 
     fn result(&mut self) -> Result<R, Self::Error>;
 }
+
+/////
+///// `std` features.
+/////
+
+use std::fmt;
+
+impl fmt::Display for Var {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        write!(f, "%{}", self.get_id())
+    }
+}
+
+impl<I> fmt::Display for Operator<I>
+where
+    I: fmt::Display,
+{
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        match self {
+            Operator::Intrinsic(v) => write!(f, "{}", v),
+            Operator::ModuleRef(module, name) => match module {
+                None => write!(f, "@{}", name),
+                Some(v) => write!(f, "{}.@{}", v, name),
+            },
+        }
+    }
+}
+
+impl fmt::Display for Branch {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        write!(f, "  br {} (", self.block)?;
+        let l = self.args.len();
+        for (ind, arg) in self.args.iter().enumerate() {
+            match l - 1 == ind {
+                true => write!(f, "{}", arg)?,
+                _ => write!(f, "{}, ", arg)?,
+            };
+        }
+        write!(f, ")")?;
+        match self.cond {
+            None => Ok(()),
+            Some(v) => write!(f, " if {}", v),
+        }
+    }
+}
+
+impl<I, A> fmt::Display for Instruction<I, A>
+where
+    I: fmt::Display,
+    A: fmt::Display,
+{
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        write!(f, "{}", self.op)?;
+        write!(f, "(")?;
+        let l = self.args.len();
+        for (ind, arg) in self.args.iter().enumerate() {
+            match l - 1 == ind {
+                true => write!(f, "{}", arg)?,
+                _ => write!(f, "{}, ", arg)?,
+            };
+        }
+        write!(f, ")")?;
+        if !self.attrs.is_empty() {
+            write!(f, " {{ ")?;
+            let l = self.attrs.len();
+            for (ind, attr) in self.attrs.iter().enumerate() {
+                match l - 1 == ind {
+                    true => write!(f, "{}", attr)?,
+                    _ => write!(f, "{}, ", attr)?,
+                };
+            }
+            write!(f, " }}")?;
+        }
+        Ok(())
+    }
+}
+
+impl<I, A> fmt::Display for ExtIR<I, A>
+where
+    I: fmt::Display,
+    A: fmt::Display,
+{
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        for ind in 0..self.blocks.len() {
+            write!(f, "{}: ", ind)?;
+            let b = &self.blocks[ind];
+            let bargs = &b.args;
+            if !bargs.is_empty() {
+                write!(f, "(")?;
+                let l = bargs.len();
+                for (ind, arg) in bargs.iter().enumerate() {
+                    match l - 1 == ind {
+                        true => write!(f, "{}", arg)?,
+                        _ => write!(f, "{}, ", arg)?,
+                    };
+                }
+                write!(f, ")")?;
+            }
+            writeln!(f)?;
+            for (v, instr) in self.block_iter(ind) {
+                writeln!(f, "  {} = {}", v, instr)?;
+            }
+            for br in &self.blocks[ind].branches {
+                writeln!(f, "{}", br)?;
+            }
+        }
+        Ok(())
+    }
+}
